import os
import re
import sys

if sys.platform.startswith('win'):
    is_win = True
    dir_char = '\\'
else:
    is_win = False
    dir_char = '/'
work_dir = os.getcwd()
work_project = work_dir.split(dir_char)[-1]
work_dir += dir_char

cpp_template = """#include <iostream>
using namespace std;

int main(int argc, char **argv) {

    return 0;
}
"""

c_template = """#include <stdio.h>

int main(int argc, char **argv) {

    return 0;
}
"""


def get_config():
    config = {}
    try:
        with open('project_configure.csv', 'r') as f:
            for row in f.readlines():
                row = row.split(',')
                config[row[0]] = [i.strip() for i in row[1:]]
            for i in config:
                if i != 'compile_tool':
                    config[i] = config[i][0]
    except IOError:
        exit("No file named: project_configure.csv\n May you need run:\"Qpro -init\" first!")
    return config


def scp_init(server_target, ct=False):
    if server_target:
        if ct:
            st = os.system('scp -r %s %s' % (work_project, server_target))
        else:
            st = os.system('scp -r * %s' % server_target)
        if st:
            exit("upload project failed!")


def create():
    try:
        project_name = sys.argv[sys.argv.index('-c') + 1]
        global work_project
        work_project = project_name
    except IndexError:
        exit('usage: Qpro -c project')
    else:
        if os.path.exists(project_name) and os.path.isdir(project_name):
            exit('"%s" is exist!' % (work_dir + project_name + dir_char))
        os.mkdir(project_name)
        is_cpp = True if input('Is a cpp project? [y/n]:') == 'y' else False
        server_target = input('input [user@ip:dir_path] if you need scp:')
        if server_target and not server_target.endswith('/') and not server_target.endswith(':'):
            server_target += '/'
        source_file = 'main.cpp' if is_cpp else 'main.c'
        with open(project_name + dir_char + source_file, 'w') as f:
            f.write(cpp_template if is_cpp else c_template)
        os.mkdir(project_name + dir_char + 'dist')
        os.mkdir(project_name + dir_char + 'template')
        info = [
            ['compile_tool', 'g++ -std=c++11' if is_cpp else 'gcc -std=c11', ''],
            ['compile_filename', source_file],
            ['executable_filename', 'dist' + dir_char + project_name],
            ['input_file', 'dist' + dir_char + 'input.txt'],
            ['template_root', 'template' + dir_char],
            ['server_target', server_target]
        ]
        with open(project_name + dir_char + 'project_configure.csv', 'w') as f:
            for row in info:
                f.write(','.join(row) + '\n')
        with open(project_name + dir_char + info[3][-1], 'w') as f:
            f.write('edit this file to make input')
        with open(project_name + dir_char + info[1][-1], 'r') as f:
            main_cont = f.read()
        with open(project_name + dir_char + 'template' + dir_char + 'main', 'w') as f:
            f.write(main_cont)
        scp_init(server_target, True)


def scp():
    try:
        path = sys.argv[sys.argv.index('-scp') + 1]
    except IndexError:
        exit('usage: Qpro -scp file')
    else:
        if not os.path.abspath(path).startswith(work_dir):
            exit("%s is not in this Qpro project!" % path)
        if not os.path.exists(path):
            exit('No such file named: ' + path)
        config = get_config()
        if os.path.isdir(path):
            os.system('scp -r %s %s' % (path, config['server_target'] + path))
        else:
            os.system('scp %s %s' % (path, config['server_target'] + path))


def get():
    try:
        path = sys.argv[sys.argv.index('-get') + 1]
    except IndexError:
        exit('usage: Qpro -get file')
    else:
        if not os.path.abspath(path).startswith(work_dir):
            exit("%s is not in this Qpro project!" % path)
        config = get_config()
        os.system('scp -r %s %s' % (config['server_target'] + path, path))


def adjust():
    config = get_config()
    import tkinter as tk
    win = tk.Tk()
    win.title('Qpro项目调整器')
    key_to_name = {
        'compile_tool': '运行指令:',
        'compile_filename': '源程序:',
        'executable_filename': '项目地址:',
        'input_file': '输入文件:',
        'template_root': '模板目录:',
        'server_target': '远程映射:'
    }
    all_dt = {}
    for i, v in enumerate(config):
        tk.Label(win, text='%12s' % key_to_name[v]).grid(row=i, column=0)
        if v == 'compile_tool':
            stringvar1 = tk.Variable()
            stringvar1.set(config[v][0])
            stringvar2 = tk.Variable()
            stringvar2.set(config[v][1])
            tk.Entry(win, textvariable=stringvar1, width=20).grid(row=i, column=1)
            tk.Entry(win, textvariable=stringvar2, width=19).grid(row=i, column=2)
            all_dt[v] = [stringvar1, stringvar2]
        else:
            stringvar1 = tk.Variable()
            stringvar1.set(config[v])
            tk.Entry(win, textvariable=stringvar1, width=40).grid(row=i, column=1, columnspan=2)
            all_dt[v] = stringvar1

    def deal_config():
        for dt in all_dt:
            if dt == 'compile_tool':
                config[dt] = [all_dt[dt][0].get(), all_dt[dt][1].get()]
            elif dt == 'server_target':
                config[dt] = all_dt[dt].get()
                if ':' in config[dt] and not config[dt].endswith('/') and not config[dt].endswith(':'):
                    config[dt] += '/'
                elif ':' not in config[dt]:
                    print('Not a legal server target!\n'
                          'You can run "Qpro -adjust" to adjust target\n'
                          'and run "Qpro -scp-init" to upload project.')
            else:
                config[dt] = all_dt[dt].get()
        if not config['template_root'].endswith(dir_char):
            config['template_root'] += dir_char
        win.destroy()
        with open('project_configure.csv', 'w') as file:
            for line in config:
                if line == 'compile_tool':
                    file.write('%s,%s\n' % (line, ','.join(config[line])))
                else:
                    file.write('%s,%s\n' % (line, config[line]))

    tk.Button(win, text='确认', command=deal_config, width=10).grid(row=6, column=0, columnspan=3)
    tk.mainloop()


def pro_init():
    if not os.path.exists('CMakeLists.txt'):
        ask = input('Not an CLion Project!' + 'You need make configure manually [y/n]:').strip()
        if 'y' not in ask and 'Y' not in ask:
            exit(0)
        is_cpp = True if input('Is a cpp project? [y/n]:').strip() == 'y' else False
        source_file = 'main.c' + ('pp' if is_cpp else '')
        while not os.path.exists(source_file):
            source_file = input('Not found "%s", set compile_filename:' % source_file).strip()
        global work_project
        while not work_project:
            work_project = input('You need to set project name:').strip()
        if not os.path.exists('dist') or not os.path.isdir('dist'):
            os.mkdir('dist')
        project_name = 'dist' + dir_char + work_project
    else:
        with open("CMakeLists.txt", 'r') as f:
            content = f.read()
        project_name = re.findall('project\((.*?)\)', content)[0].split()[0]
        if '.cpp' in content or '.CPP' in content:
            is_cpp = True
        else:
            is_cpp = False
        print('Project name:%s(%s)' % (project_name, 'CPP' if is_cpp else 'C'))
        project_name = 'cmake-build-debug' + dir_char + project_name
        source_file = 'main.c' + ('pp' if is_cpp else '')
    pro_root = dir_char.join(project_name.split(dir_char)[:-1])
    default_input = pro_root + dir_char + 'input.txt'
    server_target = input('input [user@ip:dir_path] if you need scp:')
    if ':' in server_target and not server_target.endswith('/') and not server_target.endswith(':'):
        server_target += '/'
    elif server_target:
        print('Not a legal server target!\n'
              'You can run "Qpro -adjust" to adjust target\n'
              'and run "Qpro -scp-init" to upload project.')
    print('adding project_configure')
    info = [
        ['compile_tool', 'g++ -std=c++11' if is_cpp else 'gcc -std=c11', ''],
        ['compile_filename', source_file],
        ['executable_filename', project_name],
        ['input_file', default_input],
        ['template_root', 'template' + dir_char],
        ['server_target', server_target]
    ]
    with open('project_configure.csv', 'w') as f:
        for row in info:
            f.write(','.join(row) + '\n')
    with open(info[3][-1], 'w') as f:
        f.write('edit this file to make input')
    if not os.path.exists('template') or not os.path.isdir('template'):
        os.mkdir('template')
    with open(info[1][-1], 'r') as f:
        main_cont = f.read()
    with open('template' + dir_char + 'main', 'w') as f:
        f.write(main_cont)
    scp_init(server_target)


def main():
    if '-h' in sys.argv:
        print('usage:\n'
              '\t * [Qpro -init    ]: let dir be a Qpro project!\n'
              '\t * [Qpro -h       ]: help\n'
              '\t * [Qpro -c name  ]: create a Qpro project\n'
              '\t * [Qpro -update  ]: update Qpro\n'
              '\t * [Qpro -adjust  ]: adjust configure\n'
              '\t * [Qpro -scp path]: upload path to default server target\n'
              '\t * [Qpro -scp-init]: upload all of project to server target\n'
              '\t * [Qpro -get path]: download file from server target\n'
              '\t * [tmpm *        ]: manage your template\n'
              '\t * [run *         ]: run your Qpro project\n'
              '\t * [detector -[p/f][p/f] ]: run beat detector for two source files')
    elif '-update' in sys.argv:
        os.system('pip3 install Qpro --upgrade')
        exit(0)
    elif '-c' in sys.argv:
        create()
    elif '-scp' in sys.argv:
        scp()
    elif '-get' in sys.argv:
        get()
    elif '-scp-init' in sys.argv:
        scp_init(get_config()['server_target'])
    elif '-adjust' in sys.argv:
        adjust()
    elif '-init' not in sys.argv:
        exit('wrong usage! Run "Qpro -h" for help!')
    elif not os.path.exists('project_configure.csv'):
        pro_init()
    else:
        exit("You have configured your project, see project_configure to adjust your configure!")


if __name__ == '__main__':
    main()



import sys
import os

if sys.platform.startswith('win'):
    dir_char = '\\'
else:
    dir_char = '/'
config = {}
try:
    with open('project_configure.csv', 'r') as f:
        for row in f.readlines():
            row = row.split(',')
            config[row[0]] = [i.strip() for i in row[1:]]
        for i in config:
            if i != 'compile_tool':
                config[i] = config[i][0]
        if not config['template_root'].endswith(dir_char):
            config['template_root'] += dir_char
except IOError:
    exit("No file named: project_configure.csv\n May you need run:\"Qpro -init\" first!")
is_cpp = config['compile_filename'].endswith('cpp')
algorithm_name = 'main'
temp_name = 'main'


def match_algorithm():
    with open(config['compile_filename'], 'r') as file:
        import re
        try:
            content = re.findall('/// __START__(.*?)/// __END__', file.read(), re.S)[0].strip()
        except IndexError:
            exit('No template index found! Insert "/// __START__" and "/// __END__" to your code!')
    content = content.replace('/// __TEMPLATE__', '')
    return content


def write_algorithm(temp, algorithm, content, mode):
    with open(config['template_root'] + temp, mode) as file:
        file.write('\n## %s\n\n```%s\n' % (algorithm, 'c++' if is_cpp else 'c'))
        file.write(content)
        file.write('\n```\n')


def create():
    global temp_name, algorithm_name
    indx = sys.argv.index('-c')
    try:
        temp_name = sys.argv[indx + 1]
        algorithm_name = sys.argv[indx + 2]
    except IndexError:
        if temp_name != 'main':
            with open(config['compile_filename'], 'r') as file:
                ct = file.read()
            with open(config['template_root']+temp_name, 'w') as file:
                file.write(ct)
            return
        else:
            exit('usage: tmpm -c template (algorithm)')
    temp_name += '.md'
    if os.path.exists(config['template_root'] + temp_name):
        if input('Template %s is already exist, would you cover it?[y/n]:' % temp_name) == 'n':
            exit(0)
    content = match_algorithm()
    write_algorithm(temp_name, algorithm_name, content, 'w')


def append():
    global temp_name, algorithm_name
    indx = sys.argv.index('-a')
    try:
        temp_name = sys.argv[indx + 1] + '.md'
        algorithm_name = sys.argv[indx+2]
    except IndexError:
        exit('usage: tmpm -a template algorithm')
    if os.path.exists(config['template_root'] + temp_name):
        content = match_algorithm()
        write_algorithm(temp_name, algorithm_name, content, 'a')
    else:
        sys.argv[indx] = '-c'
        create()


def join():
    global temp_name
    try:
        temp_name = sys.argv[1] + '.md'
    except IndexError:
        exit('usage: tmpm template')
    if os.path.exists(config['template_root'] + temp_name):
        with open(config['template_root'] + temp_name, 'r') as file:
            import re
            content = re.findall('##(.*?)\n.*?```.*?\n(.*?)```', file.read(), re.S)
            for i, v in enumerate(content):
                print('[%d] %s' % (i + 1, v[0].strip()), end=' ' if i + 1 % 10 else '\n')
            indx = int(input('%s选择:' % ('\n' if len(content) % 10 else ''))) - 1
            content = content[indx]
        with open(config['compile_filename'], 'r') as file:
            content = file.read().replace('/// __TEMPLATE__', content[1].strip() + '\n/// __TEMPLATE__')
        with open(config['compile_filename'], 'w') as file:
            file.write(content)
    else:
        exit('No template named: %s' % temp_name)


def h():
    print('usage:\n'
          '\t * [tmpm -h]: for help\n'
          '\t * [tmpm -r]: select copy and init\n'
          '\t * [tmpm -r backup]: init "compile_filename" to template/backup\n'
          '\t * [tmpm -c backup]: create or cover a backup\n'
          '\t * [tmpm -c template algorithm]: create template and write algorithm\n'
          '\t * [tmpm -a template algorithm]: add algorithm to template\n'
          '\t * [tmpm template]: insert algorithm in template')


def init(file_name='main'):
    with open(config['template_root'] + file_name, 'r') as file:
        content = file.read()
    with open(config['compile_filename'], 'w') as file:
        file.write(content)


def revert():
    indx = sys.argv.index('-r')
    try:
        file_name = sys.argv[indx+1]
    except IndexError:
        ls = os.listdir(config['template_root'])
        rls = []
        cnt = 1
        for i in ls:
            if '.' not in i:
                print('[%d] %s' % (cnt, i), end='\t' if cnt%8 else '\n')
                rls.append(i)
                cnt += 1
        if cnt%8:
            print()
        try:
            indx = int(input('选择:'))
            if indx<0 or indx>len(rls):
                raise IndexError
        except:
            exit('ERROR!')
        file_name = rls[indx-1]
        return init(file_name)
    if os.path.exists(config['template_root'] + file_name):
        init(file_name)
    else:
        exit('No such backup')


def main():
    if len(sys.argv) == 1:
        init()
    elif '-h' in sys.argv:
        h()
    elif '-c' in sys.argv:
        create()
    elif '-a' in sys.argv:
        append()
    elif '-r' in sys.argv:
        revert()
    else:
        join()


if __name__ == '__main__':
    main()



import os
import sys
import pyperclip

if sys.platform.startswith('win'):
    is_win = True
    dir_char = '\\'
else:
    is_win = False
    dir_char = '/'

config = {}
try:
    with open('project_configure.csv', 'r') as f:
        for row in f.readlines():
            row = row.split(',')
            config[row[0]] = [i.strip() for i in row[1:]]
        for i in config:
            if i != 'compile_tool':
                config[i] = config[i][0]
        if not config['template_root'].endswith(dir_char):
            config['template_root'] += dir_char
except IOError:
    exit("No file named: project_configure.csv\n May you need run:\"Qpro -init\" first!")
argv = []


def run(use_txt=False, executable_file=str(config['executable_filename'])):
    cmd = executable_file + ' '
    if argv:
        cmd += ' '.join(argv)
    cmd += (' < ' + config['input_file'] if use_txt else '')
    os.system(cmd)


def red_col(string):
    if is_win:
        return string
    return '\033[1;31m' + string + '\033[0m'


def blue_col(string):
    if is_win:
        return string
    return '\033[1;34m' + string + '\033[0m'


def main():
    to_build = '-b' in sys.argv or '-br' in sys.argv
    to_run = '-r' in sys.argv or '-b' not in sys.argv
    filename = config['compile_filename']
    flag = False
    if '-debug' in sys.argv:
        raise ImportError
    if '-h' in sys.argv:
        print(blue_col('usage: run.py:\n') +
              '  * ' + blue_col('build or run:\n') +
              '    # ' + red_col('[ -b ]') + ' : ' + blue_col('build\n') +
              '    # ' + red_col('[ -r ]') + ' : ' + blue_col('run\n') +
              '    # ' + red_col('[ -br]') + ' : ' + blue_col('build and run\n') +
              blue_col('    (it will run if neither of commands in "build or run")\n') +
              '  * ' + red_col('[ -i ]') + ' : ' + blue_col('use input.txt as input\n') +
              '  * ' + red_col('[ -if  *.* ]') + ' : ' + blue_col('set input file(*.*) as input\n') +
              '  * ' + red_col('[-if -paste]') + ' : ' + blue_col('use Clipboard content as input\n') +
              '  * ' + red_col('[ -f  *.cpp]') + ' : ' + blue_col('set build file as *.cpp\n') +
              '  * ' + red_col('[ -h ]') + ' : ' + blue_col('help\n') +
              '  * ' + red_col('[ *  ]') + ' : ' + blue_col('add parameters for program\n') +
              '  * ' + blue_col('Modify config to adjust default configuration'))
        exit(0)
    if '-f' in sys.argv:
        index = sys.argv.index('-f')
        if index == len(sys.argv) - 1:
            print(red_col('ERROR: No file with -f'))
        filename = sys.argv[index + 1]
        if not os.path.exists(filename):
            print(red_col('ERROR: No such file:%s' % filename))
            exit(-1)
        flag = True
    if '-if' in sys.argv:
        index = sys.argv.index('-if')
        if index == len(sys.argv) - 1:
            print(red_col('ERROR: No file with -if'))
        tmp_file = sys.argv[index + 1]
        if tmp_file == '-paste':
            with open('cmake-build-debug' + dir_char + 'input.txt', 'w') as file:
                file.write(pyperclip.paste())
        else:
            __input_file__ = tmp_file
            if not os.path.exists(__input_file__):
                print(red_col('ERROR: No such file:%s' % __input_file__))
                exit(-1)
            config['input_file'] = __input_file__
    o_file = config['executable_filename']
    if config['compile_tool'][0] and to_build:
        clang = ['clang', 'gcc', 'g++']
        use_lang = config['compile_tool'][0].split()[0]
        if use_lang in clang:
            if flag:
                o_file = filename.split(dir_char)[-1].split('.')[0]
                o_file = os.path.abspath(o_file)
            os.system(config['compile_tool'][0] + ' ' + filename + ' -o ' + o_file + ' ' + config['compile_tool'][1])
        else:  # java
            if flag:
                o_file = filename.split(dir_char)[-1].split('.')[0]
                record_file_name = o_file
                o_file = ' '.join(config['executable_filename'].split()[:-1]) + ' ' + o_file
            os.system(config['compile_tool'][0] + ' ' + filename + ' ' + config['compile_tool'][1])
    if to_run:
        add_flag = True
        for i in sys.argv[1:]:
            if not add_flag:
                add_flag = True
                continue
            if not i.startswith('-'):
                argv.append(i)
            elif i == '-if' or i == '-f':
                add_flag = False
        run('-i' in sys.argv or '-if' in sys.argv, o_file)
    if config['compile_tool'][0] and flag:
        if config['compile_tool'][0].split()[0] == 'javac':
            os.remove('dist' + dir_char + record_file_name + '.class')
        else:
            os.remove(o_file)


if __name__ == '__main__':
    main()
